from commandlib import CommandPath
from pathquery import pathquery
from path import Path
from copy import copy
import hitchbuild


class VirtualenvBuild(hitchbuild.HitchBuild):
    def __init__(self, build_path, base_python):
        self.build_path = Path(build_path).abspath()
        self.fingerprint_path = self.build_path / "fingerprint.txt"
        self.base_python = self.dependency(base_python)
        self._requirementstxt = None
        self._extra_packages = None

    @property
    def bin(self):
        return CommandPath(self.build_path / "bin")

    def with_packages(self, *packages):
        new_venv = copy(self)
        # TODO: Prevent accidental use of self instead of new_venv
        new_venv._extra_packages = new_venv.variable("extra_packages", list(packages))
        return new_venv

    def with_requirementstxt(self, *paths):
        new_venv = copy(self)
        # TODO: Prevent accidental use of self instead of new_venv
        new_venv._requirementstxt = new_venv.source(
            "requirementstxt",
            [Path(path).abspath() for path in paths]
        )
        return new_venv

    def clean(self):
        self.build_path.rmtree(ignore_errors=True)

    def build(self):
        if self.incomplete():
            self.build_path.rmtree(ignore_errors=True)
            self.build_path.mkdir()
            self.base_python.build.ensure_built()
            self.base_python.build.bin.virtualenv(self.build_path).run()

            if self._requirementstxt is not None:
                for filename in self._requirementstxt._filenames:
                    self.bin.pip("install", "-r", filename).run()

            if self._extra_packages is not None:
                for package in self._extra_packages.value:
                    self.bin.pip("install", package).run()

            self.verify()
            self.refingerprint()
        else:
            if self._requirementstxt is not None:
                if self._requirementstxt.changed:
                    for filename in self._requirementstxt._filenames:
                        self.bin.pip("install", "-r", filename).run()
                    self.refingerprint()

            if self._extra_packages is not None:
                if self._extra_packages.changed:
                    for package in self._extra_packages.value:
                        self.bin.pip("install", package).run()
                    self.refingerprint()

    def verify(self):
        assert self.base_python.build.version.replace("-dev", "") in self.bin.python(
            "-c", "import sys ; sys.stdout.write(sys.version)"
        ).output()


class PyLibrary(VirtualenvBuild):
    def __init__(self, name, base_python, module_name, library_src):
        self.base_python = self.as_dependency(base_python)
        self._module_name = module_name
        self._library_src_path = Path(library_src).abspath()
        self._library_src = self.from_source(
            "library",
            list(pathquery(self._library_src_path.joinpath(module_name))) +
            [self._library_src_path.joinpath("setup.py"), ]
        )
        self._name = name
        self._packages = []
        self._requirementstxt = None
        self._package_monitor = None

    def build(self):
        super(PyLibrary, self).build()
        if self._library_src.changes:
            self.bin.pip("uninstall", "-y", self._module_name).ignore_errors().run()
            self.bin.pip("install", ".").in_dir(self._library_src_path).run()



# -*- coding: utf-8 -*
from setuptools.command.install import install
from setuptools import find_packages
from setuptools import setup
from sys import version_info, stderr, exit
import codecs
import sys
import os


def read(*parts):
    # intentionally *not* adding an encoding option to open
    # see here: https://github.com/pypa/virtualenv/issues/201#issuecomment-3145690
    return codecs.open(os.path.join(os.path.abspath(os.path.dirname(__file__)), *parts), 'r').read()


setup(name="hitchbuildpy",
      version=read('VERSION').replace('\n', ''),
      description="Tool to build versions of python and virtualenvs.",
      long_description=read('README.rst'),
      classifiers=[
          'Development Status :: 4 - Beta',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: MIT License',
          'Topic :: Text Processing :: Markup',
          'Topic :: Software Development :: Libraries',
          'Natural Language :: English',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.1',
          'Programming Language :: Python :: 3.2',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
      ],
      keywords='virtualenv build python pyenv',
      author='Colm O\'Connor',
      author_email='colm.oconnor.github@gmail.com',
      url='https://github.com/hitchtest/hitchbuildpy',
      license='MIT',
      install_requires=["hitchbuild>=0.5.1", "pathquery>=0.2.3", "commandlib"],
      packages=find_packages(exclude=["hitch", "docs", ]),
      package_data={},
      zip_safe=False,
      include_package_data=True,
)



from commandlib import CommandPath, Command
from distutils.version import LooseVersion
from path import Path
import hitchbuild


class PyenvBuild(hitchbuild.HitchBuild):
    def __init__(self, build_path, version):
        self.build_path = Path(build_path).abspath()
        self.version = version
        self.fingerprint_path = self.build_path / "fingerprint.txt"

    @property
    def bin(self):
        return CommandPath(self.build_path / "bin")

    def clean(self):
        self.build_path.rmtree(ignore_errors=True)

    def build(self):
        if self.incomplete():
            self.build_path.rmtree(ignore_errors=True)
            self.build_path.mkdir()

            Command(
                Path(__file__).dirname().abspath().joinpath("bin", "python-build")
            )(self.version, self.build_path).run()

            if LooseVersion(self.version) < LooseVersion("3.6.0"):
                self.bin.easy_install("--upgrade", "setuptools").run()
                self.bin.easy_install("--upgrade", "pip").run()
            else:
                self.bin.pip("install", "pip", "--upgrade").run()
            self.bin.pip("install", "virtualenv", "--upgrade")\
                    .without_env("PIP_REQUIRE_VIRTUALENV").run()
            self.verify()
            self.refingerprint()

    def verify(self):
        version_to_check = self.version.replace("-dev", "")
        assert version_to_check in self.bin.python("--version").output(), \
            "'{0}' expected to be found in python --version, got:\n{1}".format(
                self.version,
                self.bin.python("--version").output(),
            )
